@@ -1,18 +1,19 @@
 import copy
 
-from woodwork.accessor_utils import _is_dataframe, _is_series
-from woodwork.utils import import_or_none
-
-dd = import_or_none('dask.dataframe')
-ks = import_or_none('databricks.koalas')
+from woodwork.accessor_utils import (
+    _is_dask_dataframe,
+    _is_dask_series,
+    _is_dataframe,
+    _is_series,
+)
 
 
 class _iLocIndexer:
     def __init__(self, data):
         self.data = data
-        if dd and isinstance(self.data, dd.DataFrame):
+        if _is_dask_dataframe(data):
             raise TypeError("iloc is not supported for Dask DataFrames")
-        elif dd and isinstance(data, dd.Series):
+        elif _is_dask_series(data):
             raise TypeError("iloc is not supported for Dask Series")
 
     def __getitem__(self, key):
@@ -31,12 +32,14 @@ class _locIndexer:
 
 def _process_selection(selection, original_data):
     if _is_series(selection):
-        if dd and isinstance(selection, dd.Series):
+        if _is_dask_series(selection):
             # Dask index values are a delayed object - can't compare below without computing
             index_vals = selection.index.values.compute()
         else:
             index_vals = selection.index.values
-        if _is_dataframe(original_data) and set(index_vals) == set(original_data.columns):
+        if _is_dataframe(original_data) and set(index_vals) == set(
+            original_data.columns
+        ):
             # Selecting a single row from a DataFrame, returned as Series without Woodwork initialized
             schema = None
         elif _is_dataframe(original_data):
@@ -51,6 +54,6 @@ def _process_selection(selection, original_data):
         # Selecting a new DataFrame from an existing DataFrame
         schema = original_data.ww.schema
         new_schema = schema._get_subset_schema(list(selection.columns))
-        selection.ww.init(schema=new_schema, validate=False)
+        selection.ww.init_with_full_schema(schema=new_schema, validate=False)
     # Selecting a single value or return selection from above
     return selection
