@@ -1,34 +1,35 @@
-
 import woodwork as ww
+from woodwork.accessor_utils import _is_koalas_series
 from woodwork.logical_types import (
     Boolean,
     BooleanNullable,
     Categorical,
     Datetime,
     Double,
+    EmailAddress,
     Integer,
     IntegerNullable,
     LogicalType,
-    NaturalLanguage,
-    Timedelta
+    Timedelta,
+    Unknown,
 )
 from woodwork.tests.testing_utils import to_pandas
 from woodwork.utils import import_or_none
 
 UNSUPPORTED_KOALAS_DTYPES = [
-    'int32',
-    'intp',
-    'uint8',
-    'uint16',
-    'uint32',
-    'uint64',
-    'uintp',
-    'float_',
-    'object',
-    'category',
+    "int32",
+    "intp",
+    "uint8",
+    "uint16",
+    "uint32",
+    "uint64",
+    "uintp",
+    "float_",
+    "object",
+    "category",
 ]
 
-ks = import_or_none('databricks.koalas')
+ks = import_or_none("databricks.koalas")
 
 
 def get_koalas_dtypes(dtypes):
@@ -36,8 +37,8 @@ def get_koalas_dtypes(dtypes):
 
 
 def test_integer_inference(integers):
-    dtypes = ['int8', 'int16', 'int32', 'int64', 'intp', 'int', 'Int64']
-    if ks and isinstance(integers[0], ks.Series):
+    dtypes = ["int8", "int16", "int32", "int64", "intp", "int", "Int64"]
+    if _is_koalas_series(integers[0]):
         dtypes = get_koalas_dtypes(dtypes)
 
     for series in integers:
@@ -47,8 +48,8 @@ def test_integer_inference(integers):
 
 
 def test_double_inference(doubles):
-    dtypes = ['float', 'float32', 'float64', 'float_']
-    if ks and isinstance(doubles[0], ks.Series):
+    dtypes = ["float", "float32", "float64", "float_"]
+    if _is_koalas_series(doubles[0]):
         dtypes = get_koalas_dtypes(dtypes)
 
     for series in doubles:
@@ -58,7 +59,7 @@ def test_double_inference(doubles):
 
 
 def test_boolean_inference(bools):
-    dtypes = ['bool', 'boolean']
+    dtypes = ["bool", "boolean"]
 
     for series in bools:
         for dtype in dtypes:
@@ -71,8 +72,8 @@ def test_boolean_inference(bools):
 
 
 def test_datetime_inference(datetimes):
-    dtypes = ['object', 'string', 'datetime64[ns]']
-    if ks and isinstance(datetimes[0], ks.Series):
+    dtypes = ["object", "string", "datetime64[ns]"]
+    if _is_koalas_series(datetimes[0]):
         dtypes = get_koalas_dtypes(dtypes)
 
     for series in datetimes:
@@ -81,9 +82,31 @@ def test_datetime_inference(datetimes):
             assert isinstance(inferred_type, Datetime)
 
 
+def test_email_inference(emails):
+    dtypes = ["object", "string"]
+    if _is_koalas_series(emails[0]):
+        dtypes = get_koalas_dtypes(dtypes)
+
+    for series in emails:
+        for dtype in dtypes:
+            inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
+            assert isinstance(inferred_type, EmailAddress)
+
+
+def test_email_inference_failure(bad_emails):
+    dtypes = ["object", "string"]
+    if _is_koalas_series(bad_emails[0]):
+        dtypes = get_koalas_dtypes(dtypes)
+
+    for series in bad_emails:
+        for dtype in dtypes:
+            inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
+            assert not isinstance(inferred_type, EmailAddress)
+
+
 def test_categorical_inference(categories):
-    dtypes = ['object', 'string', 'category']
-    if ks and isinstance(categories[0], ks.Series):
+    dtypes = ["object", "string", "category"]
+    if _is_koalas_series(categories[0]):
         dtypes = get_koalas_dtypes(dtypes)
     for series in categories:
         for dtype in dtypes:
@@ -91,66 +114,75 @@ def test_categorical_inference(categories):
             assert isinstance(inferred_type, Categorical)
 
 
+def test_categorical_inference_based_on_dtype(categories_dtype):
+    """
+    This test specifically targets the case in which a series can be inferred
+    to be categorical strictly from its pandas dtype, but would otherwise be
+    inferred as some other type.
+    """
+    inferred_type = ww.type_system.infer_logical_type(categories_dtype["cat"])
+    assert isinstance(inferred_type, Categorical)
+
+    inferred_type = ww.type_system.infer_logical_type(categories_dtype["non_cat"])
+    assert not isinstance(inferred_type, Categorical)
+
+
 def test_categorical_integers_inference(integers):
-    ww.config.set_option('numeric_categorical_threshold', 10)
-    dtypes = ['int8', 'int16', 'int32', 'int64', 'intp', 'int', 'Int64']
-    if ks and isinstance(integers[0], ks.Series):
-        dtypes = get_koalas_dtypes(dtypes)
-    for series in integers:
-        for dtype in dtypes:
-            inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
-            assert isinstance(inferred_type, Categorical)
-    ww.config.reset_option('numeric_categorical_threshold')
+    with ww.config.with_options(numeric_categorical_threshold=0.5):
+        dtypes = ["int8", "int16", "int32", "int64", "intp", "int", "Int64"]
+        if _is_koalas_series(integers[0]):
+            dtypes = get_koalas_dtypes(dtypes)
+        for series in integers:
+            for dtype in dtypes:
+                inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
+                assert isinstance(inferred_type, Categorical)
 
 
 def test_categorical_double_inference(doubles):
-    ww.config.set_option('numeric_categorical_threshold', 10)
-    dtypes = ['float', 'float32', 'float64', 'float_']
-    if ks and isinstance(doubles[0], ks.Series):
-        dtypes = get_koalas_dtypes(dtypes)
-    for series in doubles:
-        for dtype in dtypes:
-            inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
-            assert isinstance(inferred_type, Categorical)
-    ww.config.reset_option('numeric_categorical_threshold')
+    with ww.config.with_options(numeric_categorical_threshold=0.5):
+        dtypes = ["float", "float32", "float64", "float_"]
+        if _is_koalas_series(doubles[0]):
+            dtypes = get_koalas_dtypes(dtypes)
+        for series in doubles:
+            for dtype in dtypes:
+                inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
+                assert isinstance(inferred_type, Categorical)
 
 
 def test_timedelta_inference(timedeltas):
-    dtypes = ['timedelta64[ns]']
+    dtypes = ["timedelta64[ns]"]
     for series in timedeltas:
         for dtype in dtypes:
             inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
             assert isinstance(inferred_type, Timedelta)
 
 
-def test_natural_language_inference(strings):
-    dtypes = ['object', 'string']
-    if ks and isinstance(strings[0], ks.Series):
+def test_unknown_inference(strings):
+    dtypes = ["object", "string"]
+    if _is_koalas_series(strings[0]):
         dtypes = get_koalas_dtypes(dtypes)
 
     for series in strings:
         for dtype in dtypes:
             inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
-            assert isinstance(inferred_type, NaturalLanguage)
+            assert isinstance(inferred_type, Unknown)
 
 
-def test_natural_language_inference_with_threshhold(long_strings):
-    dtypes = ['object', 'string']
-    if ks and isinstance(long_strings[0], ks.Series):
+def test_unknown_inference_all_null(nulls):
+    dtypes = ["object", "string", "category", "datetime64[ns]"]
+    if _is_koalas_series(nulls[0]):
         dtypes = get_koalas_dtypes(dtypes)
 
-    ww.config.set_option('natural_language_threshold', 19)
-    for dtype in dtypes:
-        inferred_type = ww.type_system.infer_logical_type(long_strings[0].astype(dtype))
-        assert isinstance(inferred_type, NaturalLanguage)
-        inferred_type = ww.type_system.infer_logical_type(long_strings[1].astype(dtype))
-        assert isinstance(inferred_type, Categorical)
-    ww.config.reset_option('natural_language_threshold')
+    for series in nulls:
+        for dtype in dtypes:
+            inferred_type = ww.type_system.infer_logical_type(series.astype(dtype))
+            inferred_type.transform(series)
+            assert isinstance(inferred_type, Unknown)
 
 
 def test_pdna_inference(pdnas):
     expected_logical_types = [
-        NaturalLanguage,
+        Unknown,
         IntegerNullable,
         BooleanNullable,
     ]
@@ -165,16 +197,16 @@ def test_updated_ltype_inference(integers, type_sys):
     type_sys.remove_type(ww.logical_types.Integer)
 
     class Integer(LogicalType):
-        primary_dtype = 'string'
+        primary_dtype = "string"
 
     type_sys.add_type(Integer, inference_function=inference_fn)
 
-    dtypes = ['int8', 'int16', 'int32', 'int64', 'intp', 'int', 'Int64']
-    if ks and isinstance(integers[0], ks.Series):
+    dtypes = ["int8", "int16", "int32", "int64", "intp", "int", "Int64"]
+    if _is_koalas_series(integers[0]):
         dtypes = get_koalas_dtypes(dtypes)
 
     for series in integers:
         for dtype in dtypes:
             inferred_type = type_sys.infer_logical_type(series.astype(dtype))
             assert isinstance(inferred_type, Integer)
-            assert inferred_type.primary_dtype == 'string'
+            assert inferred_type.primary_dtype == "string"
