@@ -1,18 +1,25 @@
+import pandas as pd
+
 from .inference_functions import (
     boolean_func,
     boolean_nullable_func,
     categorical_func,
     datetime_func,
     double_func,
+    email_address_func,
     integer_func,
     integer_nullable_func,
-    timedelta_func
+    ip_address_func,
+    timedelta_func,
+    url_func,
 )
 
+from woodwork.accessor_utils import _is_dask_series, _is_koalas_series
 from woodwork.logical_types import (
     URL,
     Address,
     Age,
+    AgeFractional,
     AgeNullable,
     Boolean,
     BooleanNullable,
@@ -33,16 +40,14 @@ from woodwork.logical_types import (
     PhoneNumber,
     PostalCode,
     SubRegionCode,
-    Timedelta
+    Timedelta,
+    Unknown,
 )
-from woodwork.utils import import_or_none
-
-dd = import_or_none('dask.dataframe')
-ks = import_or_none('databricks.koalas')
 
 DEFAULT_INFERENCE_FUNCTIONS = {
     Address: None,
     Age: None,
+    AgeFractional: None,
     AgeNullable: None,
     Boolean: boolean_func,
     BooleanNullable: boolean_nullable_func,
@@ -50,12 +55,12 @@ DEFAULT_INFERENCE_FUNCTIONS = {
     CountryCode: None,
     Datetime: datetime_func,
     Double: double_func,
-    EmailAddress: None,
+    EmailAddress: email_address_func,
     Filepath: None,
     PersonFullName: None,
     Integer: integer_func,
     IntegerNullable: integer_nullable_func,
-    IPAddress: None,
+    IPAddress: ip_address_func,
     LatLong: None,
     NaturalLanguage: None,
     Ordinal: None,
@@ -63,7 +68,8 @@ DEFAULT_INFERENCE_FUNCTIONS = {
     PostalCode: None,
     SubRegionCode: None,
     Timedelta: timedelta_func,
-    URL: None
+    URL: url_func,
+    Unknown: None,
 }
 
 # (ParentType, ChildType)
@@ -73,23 +79,21 @@ DEFAULT_RELATIONSHIPS = [
     (Categorical, Ordinal),
     (Categorical, PostalCode),
     (Categorical, SubRegionCode),
+    (Double, AgeFractional),
     (Integer, Age),
     (IntegerNullable, AgeNullable),
     (IntegerNullable, Integer),
-    (NaturalLanguage, Address),
-    (NaturalLanguage, EmailAddress),
-    (NaturalLanguage, Filepath),
-    (NaturalLanguage, PersonFullName),
-    (NaturalLanguage, IPAddress),
-    (NaturalLanguage, PhoneNumber),
-    (NaturalLanguage, URL),
 ]
 
-DEFAULT_TYPE = NaturalLanguage
+DEFAULT_TYPE = Unknown
+
+INFERENCE_SAMPLE_SIZE = 100000
 
 
 class TypeSystem(object):
-    def __init__(self, inference_functions=None, relationships=None, default_type=NaturalLanguage):
+    def __init__(
+        self, inference_functions=None, relationships=None, default_type=DEFAULT_TYPE
+    ):
         """Create a new TypeSystem object. LogicalTypes that are present in the keys of
         the inference_functions dictionary will be considered registered LogicalTypes.
 
@@ -102,7 +106,7 @@ class TypeSystem(object):
                 second element should be the child LogicalType. If not specified, will default to an empty list
                 indicating all types should be considered root types with no children.
             default_type (LogicalType, optional): The default LogicalType to use if no inference matches are
-                found. If not specified, will default to the built-in NaturalLanguage LogicalType.
+                found. If not specified, will default to the built-in Unknown LogicalType.
         """
         self.default_type = default_type
         if inference_functions:
@@ -135,13 +139,17 @@ class TypeSystem(object):
         """
         if isinstance(parent, str):
             parent = self.str_to_logical_type(parent)
-        self._validate_type_input(logical_type=logical_type,
-                                  inference_function=inference_function,
-                                  parent=parent)
+        self._validate_type_input(
+            logical_type=logical_type,
+            inference_function=inference_function,
+            parent=parent,
+        )
 
         registered_ltype_names = [ltype.__name__ for ltype in self.registered_types]
         if logical_type.__name__ in registered_ltype_names:
-            raise ValueError(f'Logical Type with name {logical_type.__name__} already present in the Type System. Please rename the LogicalType or remove existing one.')
+            raise ValueError(
+                f"Logical Type with name {logical_type.__name__} already present in the Type System. Please rename the LogicalType or remove existing one."
+            )
         self.update_inference_function(logical_type, inference_function)
         if parent:
             self.update_relationship(logical_type, parent)
@@ -169,7 +177,9 @@ class TypeSystem(object):
                 self.update_relationship(child, parent)
 
         # Rebuild the relationships list to remove any reference to the removed type
-        self.relationships = [rel for rel in self.relationships if logical_type not in rel]
+        self.relationships = [
+            rel for rel in self.relationships if logical_type not in rel
+        ]
 
     def update_inference_function(self, logical_type, inference_function):
         """Update the inference function for the specified LogicalType.
@@ -181,7 +191,9 @@ class TypeSystem(object):
         """
         if isinstance(logical_type, str):
             logical_type = self.str_to_logical_type(logical_type)
-        self._validate_type_input(logical_type=logical_type, inference_function=inference_function)
+        self._validate_type_input(
+            logical_type=logical_type, inference_function=inference_function
+        )
         self.inference_functions[logical_type] = inference_function
 
     def update_relationship(self, logical_type, parent):
@@ -199,7 +211,9 @@ class TypeSystem(object):
             parent = self.str_to_logical_type(parent)
         self._validate_type_input(logical_type=logical_type, parent=parent)
         # If the logical_type already has a parent, remove that from the list
-        self.relationships = [rel for rel in self.relationships if rel[1] != logical_type]
+        self.relationships = [
+            rel for rel in self.relationships if rel[1] != logical_type
+        ]
         # Add the new/updated relationship
         self.relationships.append((parent, logical_type))
 
@@ -221,7 +235,9 @@ class TypeSystem(object):
     @property
     def root_types(self):
         """Returns a list of all registered types that do not have a parent type"""
-        return [ltype for ltype in self.registered_types if self._get_parent(ltype) is None]
+        return [
+            ltype for ltype in self.registered_types if self._get_parent(ltype) is None
+        ]
 
     def _get_children(self, logical_type):
         """List of all the child types for the given logical type"""
@@ -243,15 +259,17 @@ class TypeSystem(object):
             parent = self._get_parent(parent)
         return depth
 
-    def _validate_type_input(self, logical_type=None, inference_function=None, parent=None):
+    def _validate_type_input(
+        self, logical_type=None, inference_function=None, parent=None
+    ):
         if logical_type and logical_type not in LogicalType.__subclasses__():
-            raise TypeError('logical_type must be a valid LogicalType')
+            raise TypeError("logical_type must be a valid LogicalType")
 
         if inference_function and not callable(inference_function):
-            raise TypeError('inference_function must be a function')
+            raise TypeError("inference_function must be a function")
 
         if parent and parent not in self.registered_types:
-            raise ValueError('parent must be a valid LogicalType')
+            raise ValueError("parent must be a valid LogicalType")
 
     def infer_logical_type(self, series):
         """Infer the logical type for the given series
@@ -259,25 +277,39 @@ class TypeSystem(object):
         Args:
             series (pandas.Series): The series for which to infer the LogicalType.
         """
+        if isinstance(series, pd.Series):
+            # Special case for series with no valid values
+            if series.count() == 0:
+                return Unknown()
 
-        # Bring Dask or Koalas data into memory for inference
-        if dd and isinstance(series, dd.Series):
-            series = series.get_partition(0).compute()
-        if ks and isinstance(series, ks.Series):
-            series = series.head(100000).to_pandas()
+            series = series.head(INFERENCE_SAMPLE_SIZE)
+        else:
+            if _is_dask_series(series):
+                series = series.head(INFERENCE_SAMPLE_SIZE)
+            elif _is_koalas_series(series):
+                series = series.head(INFERENCE_SAMPLE_SIZE).to_pandas()
+            else:
+                raise ValueError(
+                    f"Unsupported series type `{type(series)}`"
+                )  # pragma: no cover
+
+            # For dask or koalas collections, unknown type special case comes
+            # *after* head calls to avoid evaluating a potentially large
+            # dataset
+            if series.count() == 0:
+                return Unknown()
 
         def get_inference_matches(types_to_check, series, type_matches=[]):
             # Since NaturalLanguage isn't inferred by default, make sure to check
             # any children of NaturalLanguage, otherwise they never get evaluated
-            if NaturalLanguage in types_to_check:
-                check_next = self._get_children(NaturalLanguage)
-            else:
-                check_next = []
+            check_next = []
             for logical_type in types_to_check:
                 inference_func = self.inference_functions.get(logical_type)
                 if inference_func and inference_func(series):
                     type_matches.append(logical_type)
                     check_next.extend(self._get_children(logical_type))
+                elif not inference_func:
+                    check_next.extend(self._get_children(logical_type))
             if len(check_next) > 0:
                 get_inference_matches(check_next, series, type_matches)
             return type_matches
@@ -285,7 +317,7 @@ class TypeSystem(object):
         type_matches = get_inference_matches(self.root_types, series)
 
         if len(type_matches) == 0:
-            # If no matches, set type to default type (NaturalLanguage)
+            # If no matches, set type to default type (Unknown)
             logical_type = self.default_type
         elif len(type_matches) == 1:
             # If we match only one type, return it
@@ -308,9 +340,15 @@ class TypeSystem(object):
     def _get_logical_types(self):
         """Returns a dictionary of logical type name strings and logical type classes"""
         # Get snake case strings
-        logical_types = {logical_type.type_string: logical_type for logical_type in self.registered_types}
+        logical_types = {
+            logical_type.type_string: logical_type
+            for logical_type in self.registered_types
+        }
         # Add class name strings
-        class_name_dict = {logical_type.__name__: logical_type for logical_type in self.registered_types}
+        class_name_dict = {
+            logical_type.__name__: logical_type
+            for logical_type in self.registered_types
+        }
         logical_types.update(class_name_dict)
 
         return logical_types
@@ -319,7 +357,10 @@ class TypeSystem(object):
         """Helper function for converting a string value to the corresponding logical type object.
         If a dictionary of params for the logical type is provided, apply them."""
         logical_str_lower = logical_str.lower()
-        logical_types_dict = {ltype_name.lower(): ltype for ltype_name, ltype in self._get_logical_types().items()}
+        logical_types_dict = {
+            ltype_name.lower(): ltype
+            for ltype_name, ltype in self._get_logical_types().items()
+        }
 
         if logical_str_lower in logical_types_dict:
             ltype = logical_types_dict[logical_str_lower]
@@ -328,9 +369,11 @@ class TypeSystem(object):
             else:
                 return ltype
         elif raise_error:
-            raise ValueError('String %s is not a valid logical type' % logical_str)
+            raise ValueError("String %s is not a valid logical type" % logical_str)
 
 
-type_system = TypeSystem(inference_functions=DEFAULT_INFERENCE_FUNCTIONS,
-                         relationships=DEFAULT_RELATIONSHIPS,
-                         default_type=DEFAULT_TYPE)
+type_system = TypeSystem(
+    inference_functions=DEFAULT_INFERENCE_FUNCTIONS,
+    relationships=DEFAULT_RELATIONSHIPS,
+    default_type=DEFAULT_TYPE,
+)
