@@ -8,17 +8,20 @@ import numpy as np
 import pandas as pd
 
 import woodwork as ww
+from woodwork.pandas_backport import guess_datetime_format
 
 # Dictionary mapping formats/content types to the appropriate pandas read function
 type_to_read_func_map = {
-    'csv': pd.read_csv,
-    'text/csv': pd.read_csv,
-    'parquet': pd.read_parquet,
-    'application/parquet': pd.read_parquet,
-    'arrow': pd.read_feather,
-    'application/arrow': pd.read_feather,
-    'feather': pd.read_feather,
-    'application/feather': pd.read_feather
+    "csv": pd.read_csv,
+    "text/csv": pd.read_csv,
+    "parquet": pd.read_parquet,
+    "application/parquet": pd.read_parquet,
+    "arrow": pd.read_feather,
+    "application/arrow": pd.read_feather,
+    "feather": pd.read_feather,
+    "application/feather": pd.read_feather,
+    "orc": pd.read_orc,
+    "application/orc": pd.read_orc,
 }
 
 PYARROW_ERR_MSG = (
@@ -30,9 +33,10 @@ PYARROW_ERR_MSG = (
 )
 
 # Add new mimetypes
-add_type('application/parquet', '.parquet')
-add_type('application/arrow', '.arrow')
-add_type('application/feather', '.feather')
+add_type("application/parquet", ".parquet")
+add_type("application/arrow", ".arrow")
+add_type("application/feather", ".feather")
+add_type("application/orc", ".orc")
 
 
 def import_or_none(library):
@@ -49,14 +53,14 @@ def import_or_none(library):
 
 
 def camel_to_snake(s):
-    s = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', s)
-    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s).lower()
+    s = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", s)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s).lower()
 
 
-def _convert_input_to_set(semantic_tags, error_language='semantic_tags', validate=True):
+def _convert_input_to_set(semantic_tags, error_language="semantic_tags", validate=True):
     """Takes input as a single string, a list of strings, or a set of strings
-        and returns a set with the supplied values. If no values are supplied,
-        an empty set will be returned."""
+    and returns a set with the supplied values. If no values are supplied,
+    an empty set will be returned."""
     if not semantic_tags:
         return set()
 
@@ -85,16 +89,20 @@ def _validate_string_tags(semantic_tags, error_language):
         raise TypeError(f"{error_language} must contain only strings")
 
 
-def read_file(filepath=None,
-              content_type=None,
-              name=None,
-              index=None,
-              time_index=None,
-              semantic_tags=None,
-              logical_types=None,
-              use_standard_tags=True,
-              validate=True,
-              **kwargs):
+def read_file(
+    filepath=None,
+    content_type=None,
+    name=None,
+    index=None,
+    time_index=None,
+    semantic_tags=None,
+    logical_types=None,
+    use_standard_tags=True,
+    column_origins=None,
+    replace_nan=False,
+    validate=True,
+    **kwargs,
+):
     """Read data from the specified file and return a DataFrame with initialized Woodwork typing information.
 
         Note:
@@ -121,6 +129,10 @@ def read_file(filepath=None,
             for any columns not present in the dictionary.
         use_standard_tags (bool, optional): If True, will add standard semantic tags to columns based
             on the inferred or specified logical type for the column. Defaults to True.
+        column_origins (str or dict[str -> str], optional): Origin of each column. If a string is supplied, it is
+                used as the origin for all columns. A dictionary can be used to set origins for individual columns.
+        replace_nan (bool, optional): Whether to replace empty string values and string representations of
+            NaN values ("nan", "<NA>") with np.nan or pd.NA values based on column dtype. Defaults to False.
         validate (bool, optional): Whether parameter and data validation should occur. Defaults to True. Warning:
                 Should be set to False only when parameters and data are known to be valid.
                 Any errors resulting from skipping validation with invalid inputs may not be easily understood.
@@ -133,25 +145,48 @@ def read_file(filepath=None,
     if content_type is None:
         inferred_type, _ = guess_type(filepath)
         if inferred_type is None:
-            raise RuntimeError('Content type could not be inferred. Please specify content_type and try again.')
+            raise RuntimeError(
+                "Content type could not be inferred. Please specify content_type and try again."
+            )
         content_type = inferred_type
 
     if content_type not in type_to_read_func_map:
-        raise RuntimeError('Reading from content type {} is not currently supported'.format(content_type))
+        raise RuntimeError(
+            "Reading from content type {} is not currently supported".format(
+                content_type
+            )
+        )
 
-    if content_type in ['parquet', 'application/parquet', 'arrow', 'application/arrow', 'feather', 'application/feather']:
-        import_or_raise('pyarrow', PYARROW_ERR_MSG)
-        if content_type in ['parquet', 'application/parquet']:
-            kwargs['engine'] = 'pyarrow'
+    pyarrow_types = [
+        "parquet",
+        "application/parquet",
+        "arrow",
+        "application/arrow",
+        "feather",
+        "application/feather",
+        "orc",
+        "application/orc",
+    ]
+    if content_type in pyarrow_types:
+        import_or_raise("pyarrow", PYARROW_ERR_MSG)
+        if content_type in ["parquet", "application/parquet"]:
+            kwargs["engine"] = "pyarrow"
 
     dataframe = type_to_read_func_map[content_type](filepath, **kwargs)
-    dataframe.ww.init(name=name,
-                      index=index,
-                      time_index=time_index,
-                      semantic_tags=semantic_tags,
-                      logical_types=logical_types,
-                      use_standard_tags=use_standard_tags,
-                      validate=validate)
+
+    if replace_nan:
+        dataframe = _replace_nan_strings(dataframe)
+
+    dataframe.ww.init(
+        name=name,
+        index=index,
+        time_index=time_index,
+        semantic_tags=semantic_tags,
+        logical_types=logical_types,
+        use_standard_tags=use_standard_tags,
+        column_origins=column_origins,
+        validate=validate,
+    )
     return dataframe
 
 
@@ -176,7 +211,7 @@ def _is_s3(string):
 
 def _is_url(string):
     """Checks if the given string is an url path. Returns a boolean."""
-    return 'http' in string
+    return "http" in string
 
 
 def _reformat_to_latlong(latlong, use_list=False):
@@ -189,17 +224,19 @@ def _reformat_to_latlong(latlong, use_list=False):
             # Serialized latlong columns from csv or parquet will be strings, so null values will be
             # read as the string 'nan' in pandas and Dask and 'NaN' in Koalas
             # neither of which which is interpretable as a null value
-            if 'nan' in latlong:
-                latlong = latlong.replace('nan', 'None')
-            if 'NaN' in latlong:
-                latlong = latlong.replace('NaN', 'None')
+            if "nan" in latlong:
+                latlong = latlong.replace("nan", "None")
+            if "NaN" in latlong:
+                latlong = latlong.replace("NaN", "None")
             latlong = ast.literal_eval(latlong)
         except ValueError:
             pass
 
     if isinstance(latlong, (tuple, list)):
         if len(latlong) != 2:
-            raise ValueError(f'LatLong values must have exactly two values. {latlong} does not have two values.')
+            raise ValueError(
+                f"LatLong values must have exactly two values. {latlong} does not have two values."
+            )
 
         latitude, longitude = map(_to_latlong_float, latlong)
 
@@ -211,7 +248,9 @@ def _reformat_to_latlong(latlong, use_list=False):
             return [latitude, longitude]
         return (latitude, longitude)
 
-    raise ValueError(f'LatLongs must either be a tuple, a list, or a string representation of a tuple. {latlong} does not fit the criteria.')
+    raise ValueError(
+        f"LatLongs must either be a tuple, a list, or a string representation of a tuple. {latlong} does not fit the criteria."
+    )
 
 
 def _to_latlong_float(val):
@@ -222,17 +261,17 @@ def _to_latlong_float(val):
     try:
         return float(val)
     except (ValueError, TypeError):
-        raise ValueError(f'Latitude and Longitude values must be in decimal degrees. The latitude or longitude represented by {val} cannot be converted to a float.')
+        raise ValueError(
+            f"Latitude and Longitude values must be in decimal degrees. The latitude or longitude represented by {val} cannot be converted to a float."
+        )
 
 
 def _is_valid_latlong_series(series):
     """Returns True if all elements in the series contain properly formatted LatLong values,
     otherwise returns False"""
-    dd = import_or_none('dask.dataframe')
-    ks = import_or_none('databricks.koalas')
-    if dd and isinstance(series, dd.Series):
-        series = series.compute()
-    if ks and isinstance(series, ks.Series):
+    if ww.accessor_utils._is_dask_series(series):
+        series = series = series.get_partition(0).compute()
+    if ww.accessor_utils._is_koalas_series(series):
         series = series.to_pandas()
         bracket_type = list
     else:
@@ -258,7 +297,7 @@ def _is_valid_latlong_value(val, bracket_type=tuple):
 
 def _is_null_latlong(val):
     if isinstance(val, str):
-        return val == 'None' or val == 'nan' or val == 'NaN'
+        return val == "None" or val == "nan" or val == "NaN"
 
     # Since we can have list inputs here, pd.isnull will not have a relevant truth value for lists
     return not isinstance(val, list) and pd.isnull(val)
@@ -278,12 +317,15 @@ def get_valid_mi_types():
     """
     valid_types = []
     for ltype in ww.type_system.registered_types:
-        if 'category' in ltype.standard_tags:
+        if "category" in ltype.standard_tags:
             valid_types.append(ltype)
-        elif 'numeric' in ltype.standard_tags:
+        elif "numeric" in ltype.standard_tags:
             valid_types.append(ltype)
-        elif (ltype == ww.logical_types.Datetime or ltype == ww.logical_types.Boolean or
-                ltype == ww.logical_types.BooleanNullable):
+        elif (
+            ltype == ww.logical_types.Datetime
+            or ltype == ww.logical_types.Boolean
+            or ltype == ww.logical_types.BooleanNullable
+        ):
             valid_types.append(ltype)
 
     return valid_types
@@ -325,13 +367,14 @@ def concat_columns(objs, validate_schema=True):
         DataFrame: A Woodwork dataframe whose typing information is also a concatenation of the input dataframes.
     """
     if not objs:
-        raise ValueError('No objects to concatenate')
+        raise ValueError("No objects to concatenate")
 
-    table_name = ''
+    table_name = ""
 
     logical_types = {}
     semantic_tags = {}
     col_descriptions = {}
+    col_origins = {}
     col_metadata = {}
     table_metadata = {}
     use_standard_tags = {}
@@ -347,14 +390,16 @@ def concat_columns(objs, validate_schema=True):
             # Raise error if there's overlap between table metadata
             overlapping_keys = obj.ww.metadata.keys() & table_metadata.keys()
             if overlapping_keys:
-                raise ValueError(f'Cannot resolve overlapping keys in table metadata: {overlapping_keys}')
+                raise ValueError(
+                    f"Cannot resolve overlapping keys in table metadata: {overlapping_keys}"
+                )
 
             table_metadata = {**obj.ww.metadata, **table_metadata}
 
             # Combine table names
             if obj.ww.name is not None:
                 if table_name:
-                    table_name += '_'
+                    table_name += "_"
                 table_name += str(obj.ww.name)
 
             # Cannot have multiple tables with indexes or time indexes set
@@ -362,14 +407,18 @@ def concat_columns(objs, validate_schema=True):
                 if index is None:
                     index = obj.ww.index
                 else:
-                    raise IndexError('Cannot set the Woodwork index of multiple input objects. '
-                                     'Please remove the index columns from all but one table.')
+                    raise IndexError(
+                        "Cannot set the Woodwork index of multiple input objects. "
+                        "Please remove the index columns from all but one table."
+                    )
             if obj.ww.time_index is not None:
                 if time_index is None:
                     time_index = obj.ww.time_index
                 else:
-                    raise IndexError('Cannot set the Woodwork time index of multiple input objects. '
-                                     'Please remove the time index columns from all but one table.')
+                    raise IndexError(
+                        "Cannot set the Woodwork time index of multiple input objects. "
+                        "Please remove the time index columns from all but one table."
+                    )
 
             ww_columns = obj.ww.schema.columns
         elif isinstance(obj.ww.schema, ww.column_schema.ColumnSchema):
@@ -378,57 +427,119 @@ def concat_columns(objs, validate_schema=True):
         # Compile the typing information per column
         for name, col_schema in ww_columns.items():
             if name in col_names_seen:
-                raise ValueError(f"Duplicate column '{name}' has been found in more than one input object. "
-                                 "Please remove duplicate columns from all but one table.")
+                raise ValueError(
+                    f"Duplicate column '{name}' has been found in more than one input object. "
+                    "Please remove duplicate columns from all but one table."
+                )
             logical_types[name] = col_schema.logical_type
-            semantic_tags[name] = col_schema.semantic_tags - {'time_index'} - {'index'}
+            semantic_tags[name] = col_schema.semantic_tags - {"time_index"} - {"index"}
             col_metadata[name] = col_schema.metadata
             col_descriptions[name] = col_schema.description
+            col_origins[name] = col_schema.origin
             use_standard_tags[name] = col_schema.use_standard_tags
 
             col_names_seen.add(name)
 
     # Perform concatenation with the correct library
     obj = objs[0]
-    dd = import_or_none('dask.dataframe')
-    ks = import_or_none('databricks.koalas')
+    dd = import_or_none("dask.dataframe")
+    ks = import_or_none("databricks.koalas")
 
     lib = pd
-    if ks and isinstance(obj, (ks.Series, ks.DataFrame)):
+    if ww.accessor_utils._is_koalas_dataframe(
+        obj
+    ) or ww.accessor_utils._is_koalas_series(obj):
         lib = ks
-    elif dd and isinstance(obj, (dd.Series, dd.DataFrame)):
+    elif ww.accessor_utils._is_dask_dataframe(obj) or ww.accessor_utils._is_dask_series(
+        obj
+    ):
         lib = dd
 
-    combined_df = lib.concat(objs, axis=1, join='outer')
+    combined_df = lib.concat(objs, axis=1, join="outer")
 
     # Initialize Woodwork with all of the typing information from the input objs
     # performing type inference on any columns that did not already have Woodwork initialized
-    combined_df.ww.init(name=table_name or None,
-                        index=index,
-                        time_index=time_index,
-                        logical_types=logical_types,
-                        semantic_tags=semantic_tags,
-                        table_metadata=table_metadata or None,
-                        column_metadata=col_metadata,
-                        column_descriptions=col_descriptions,
-                        use_standard_tags=use_standard_tags,
-                        validate=validate_schema)
+    combined_df.ww.init(
+        name=table_name or None,
+        index=index,
+        time_index=time_index,
+        logical_types=logical_types,
+        semantic_tags=semantic_tags,
+        table_metadata=table_metadata or None,
+        column_metadata=col_metadata,
+        column_descriptions=col_descriptions,
+        column_origins=col_origins,
+        use_standard_tags=use_standard_tags,
+        validate=validate_schema,
+    )
     return combined_df
 
 
-def _update_progress(start_time, current_time, progress_increment,
-                     current_progress, total, callback_function):
+def _update_progress(
+    start_time,
+    current_time,
+    progress_increment,
+    current_progress,
+    total,
+    unit,
+    callback_function,
+):
     """Helper function for updating progress of a function and making a call to the progress callback
     function, if provided. Adds the progress increment to the current progress amount and returns the
     updated progress amount.
 
     If provided, the callback function should accept the following parameters:
-        update: percentage change (float between 0 and 100) in progress since last call
-        progress_percent: percentage (float between 0 and 100) of total computation completed
-        time_elapsed: total time in seconds that has elapsed since start of call"""
+        - update (int): change in progress since last call
+        - progress (int): the progress so far in the calculations
+        - total (int): the total number of calculations to do
+        - unit (str): unit of measurement for progress/total
+        - time_elapsed (float): total time in seconds elapsed since start of call
+    """
     if callback_function is not None:
         new_progress = current_progress + progress_increment
         elapsed_time = current_time - start_time
-        callback_function((progress_increment / total) * 100, (new_progress / total) * 100, elapsed_time)
+        callback_function(progress_increment, new_progress, total, unit, elapsed_time)
 
         return new_progress
+
+
+def _infer_datetime_format(dates, n=100):
+    """Helper function to infer the datetime format of the first n non-null rows of a series
+    Args:
+        dates (Series): Series of string or datetime string to guess the format of
+        n (int): the maximum number of nonnull rows to sample from the series
+    """
+    first_n = dates.dropna().head(n)
+    if len(first_n) == 0:
+        return None
+    try:
+        fmts = first_n.map(guess_datetime_format)
+        mode_fmt = fmts.mode().loc[0]  # select first most common format
+    except (TypeError, ValueError, IndexError, KeyError, NotImplementedError):
+        mode_fmt = None
+    return mode_fmt
+
+
+def _replace_nan_strings(df):
+    """Replaces empty string values and string representations of
+    NaN values ("nan", "<NA>") with np.nan or pd.NA depending on
+    column dtype."""
+    df = df.fillna(value=np.nan)
+
+    for col, dtype in df.dtypes.items():
+        replace_val = np.nan
+        if str(dtype) == "boolean":
+            # All replace calls below fail with boolean dtype
+            # but boolean cols cannot contain strings to begin with.
+            continue
+        elif str(dtype) == "string":
+            # Must use pd.NA as replacement value for string dtype
+            replace_val = pd.NA
+
+        replaced_series = df[col].replace(r"^\s*$", replace_val, regex=True)
+        replaced_series = replaced_series.replace(
+            {"nan": replace_val, "<NA>": replace_val}
+        )
+        df[col] = replaced_series
+
+    return df
